<?php

namespace App\Http\Controllers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use App\RegTypes;
use App\Paymentlogs;
use App\User;
use Firebase\JWT\JWT;
use Firebase\JWT\ExpiredException;
use App\Member;
use App\Donation;
use App\Conference;

class AdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    private $request;

    public function __construct(Request $request)
    {
        //
        $this->request = $request;
    }

    protected function jwt(User $user) {
        $payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->id, // Subject of the token
            'iat' => time(), // Time when JWT was issued. 
            'exp' => time() + 60*60*60 // Expiration time
        ];

        return JWT::encode($payload, env('JWT_SECRET'));
    } 

    public function authenticate(User $user) {
        $this->validate($this->request, [
            'email'     => 'required|email',
            'password'  => 'required'
        ]);
        $user = User::where('email', $this->request->input('email'))->first();

        if (!$user) {
            return response()->json([
                'error' => 'Email does not exist.'
            ], 400);
        }
        if (Hash::check($this->request->input('password'), $user->password)) {
            return response()->json([
                'token' => $this->jwt($user),
                'user'=>$user
            ], 200);
        }
        return response()->json([
            'error' => 'Email or password is wrong.'
        ], 400);
    }

    public function generateKey(){
        return base64_encode(str_random(40));
    }
    public function generateRandomString($length = 12) {
        return substr(str_shuffle(str_repeat($x='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', ceil($length/strlen($x)) )),1,$length);
   }
    public function connectToDB($table){
        return DB::connection('mysql')->table($table);
    }

    public function RegTypes(){
      $data = RegTypes::get();
      return response()->json(['status' => 'success','data'=>$data]);
    }

    public function generateUniqueID(){
        $count = 0;
        $getmax = $this->connectToDB('members')->max('counter');
         if(!is_null($getmax)){
            $count = (int)$getmax;
         }
         $stepup = $count+1;
         $format = 'APBA';
         $extra = '';
         switch (strlen((string)$stepup)) {
           case 1:
             $extra = '000'.(string)$stepup;
             break;
           case 2:
             $extra = '00'.(string)$stepup;
             break;
          case 3:
             $extra = '0'.(string)$stepup;
             break;
          case 4:
             $extra = (string)$stepup;
             break;
           default:
             $extra = (string)$stepup;
             break;
         }
         $finalformat = $format.$extra;
         return ['id'=>$finalformat,'counter'=>$stepup];
       }

    public function getTitles(){
       $data = $this->connectToDB('titles')->get();
       return response()->json(['status' => 'success','data'=>$data]);
    }

    public function getCountries(){
        $data = $this->connectToDB('countries')->get();
        return response()->json(['status' => 'success','data'=>$data]);
    }

    public function members(Request $r){
      $data = Member::with('registrationType','payments')->paginate(env('PAGINATE',50));
      return response()->json(['status'=>'success','data'=>$data]);
    }

    public function donations(Request $r){
        $data = Donation::paginate(env('PAGINATE',50));
        return response()->json(['status'=>'success','data'=>$data]);
    }

    public function saveDonation(Request $r){
        $donation = new Donation;
        if($r->input('id') !== null && !empty($r->input('id'))){
            $donation  = Donation::find($r->input('id'));
        }
        $donation->title = $r->input('title');
        $donation->firstname = $r->input('firstname');
        $donation->lastname = $r->input('surname');
        $donation->email = $r->input('email');
        $donation->phone = $r->input('phone');
        $donation->comments = $r->input('comments');
        $donation->date_paid = $r->input('date_paid');
        $donation->amount = $r->input('amount');
        $donation->reference = $r->input('reference');
        $donation->transaction_id = $r->input('transaction_id');
        $donation->payment_mode = $r->input('payment_mode');
        if($donation->save()){
          return true;
        }
          return false;
    }


    public function saveConference(Request $r){
        $regtype = RegTypes::where('name',$r->input('member_type'))->first();
        $conf = new Conference;
        if($r->input('id') !== null && !empty($r->input('id'))){
            $conf  = Conference::find($r->input('id'));
        }
       // $conf->user_id = $extras->member_id;
        $conf->firstname = $r->input('firstname');
        $conf->surname = $r->input('surname');
        $conf->othernames = $r->input('othernames');
        $conf->gender = $r->input('gender');
        $conf->country = $r->input('country');
        $conf->state = $r->input('state');
        $conf->email = $r->input('email');
        $conf->phone = $r->input('phone');
        $conf->amount = $r->input('amount');
        $conf->currency = $r->input('currency');
        $conf->date_paid = $r->input('date_paid');
        $conf->reference = $r->input('reference');
        $conf->transaction_id = $r->input('transaction_id');
        $conf->payment_mode = $r->input('payment_mode');
        $conf->conftype = $r->input('paymentfrom');
       // $conf->membership_id = $regtype->id;
        $conf->save();
        return true;
    }

    public function saveMembership(Request $r){
  
      $regtype = RegTypes::where('name',$r->input('statusname'))->first();
      $genid = $this->generateUniqueID();
      $member = new Member;
      if($r->input('id') !== null && !empty($r->input('id'))){
        $member  = Member::find($r->input('id'));
       }
      $member->id = $genid['id'];
      $member->counter = $genid['counter'];
      $member->regtype_id = $regtype->id;
      $member->title = $r->input('title');
      $member->firstname = $r->input('firstname');
      $member->lastname = $r->input('lastname');
      $member->email = $r->input('email');
      $member->phone = $r->input('phone');
      $member->dob = $extras->dob;
      $member->country = $r->input('country');
      $member->state = $r->input('state');
      $member->reference = $r->input('reference');
      $member->transaction_id = $r->input('transaction_id');
      $member->payment_mode = $r->input('payment_mode');
      $member->amount = $r->input('amount');
      $member->currency = $r->input('currency');
      $member->date_paid = $r->input('date_paid');
      $member->save();
      return true;
    }

    public function addRegistrationType(Request $r){
        $this->validate($r, [
            'name' => 'required',
            'amount' => 'required',
            'currency' => 'required'
        ]);
        $regtype = new RegTypes;
        $regtype->name = $r->input('name');
        $regtype->amount = $r->input('amount');
        $regtype->currency = $r->input('currency');
        $regtype->symbol = $r->input('symbol');
        if($regtype->save()){
            return response()->json(['status'=>'success','message'=>'New Membership Type added successfully','data'=>$regtype]);
        }
        return response()->json(['status'=>'error','message'=>'Error whiles adding new Membership Type.. Please try again']);
    }

    public function RegistrationType(Request $r){
       $types = RegTypes::all();
       return response()->json(['status'=>'success','data'=>$types]);
    }

    public function deleteRegistrationType(Request $r){
        $this->validate($r, [
            'id' => 'required'
        ]);
        $find = RegTypes::find($r->input('id'));
        if(is_null($find)){
            return response()->json(['status'=>'error','message'=>'No Regtype matches the provided ID']);
        }
        $find->delete();
        return response()->json(['status'=>'success','message'=>'Item deleted successfully']);
    }

    public function editRegistrationType(Request $r){
        $this->validate($r, [
            'id' => 'required'
        ]);
        $find = RegTypes::find($r->input('id'));
        if(is_null($find)){
            return response()->json(['status'=>'error','message'=>'No Regtype matches the provided ID']);
        }
        $find->name = $r->input('name');
        $find->amount = $r->input('amount');
        $find->currency = $r->input('currency');
        $find->symbol = $r->input('symbol');
        $find->save();
        return response()->json(['status'=>'success','message'=>'Item deleted successfully']);
    }

    public function addNewUser(Request $r){
        $this->validate($r, [
            'username' => 'required',
            'email' => 'required|email|unique:users',
            'password' => 'required'
        ]);
    
       $user = new User;
       $user->username = $r->input('username');
       $user->email = $r->input('email');
       $user->password = Hash::make($r->input('password'));
       $user->save();
       return response()->json(['status'=>'success','message'=>'New User Added Successfully']);
    }

    public function deleteUser(Request $r){
        $this->validate($r, [
            'id' => 'required'
        ]);
        $find = User::find($r->input('id'));
        if(is_null($find)){
            return response()->json(['status'=>'error','message'=>'No User the provided ID']);
        }
        $find->delete();
        return response()->json(['status'=>'success','message'=>'User deleted successfully']);
    }

    public function todayData(Request $r){
        $members = Member::where('created_at','like',date('Y-m-d').'%')->get();
        $donations = Donation::where('created_at','like',date('Y-m-d').'%')->get();
        $conference = Conference::where('created_at','like',date('Y-m-d').'%')->get();
        return response()->json([
         'status'=>'success',
         'members'=>$members,
         'donations'=>$donations,
         'conference'=>$conference
        ]);
    }

    public function addMember(Request $r){
        $this->validate($r, [
            'firstname' => 'required',
            'lastname' => 'required',
            'email' => 'required',
            'phone' => 'required',
            'regtype_id' => 'required'
        ]);
       $member = new Member;
       $member->regtype_id = $r->input('regtype_id');
       $member->title = $r->input('title');
       $member->firstname = $r->input('firstname');
       $member->lastname = $r->input('lastname');
       $member->email = $r->input('email');
       $member->phone = $r->input('phone');
       $member->dob = $r->input('dob');
       $member->country = $r->input('country');
       $member->state = $r->input('state');
       $member->save();
       return response()->json(['status'=>'success','message'=>'New User Added Successfully']);
    }

    public function editMember(Request $r){
        $this->validate($r, [
            'firstname' => 'required',
            'lastname' => 'required',
            'email' => 'required',
            'phone' => 'required',
            'regtype_id' => 'required'
        ]);
       $member = Member::find($r->input('id'));
       $member->regtype_id = $r->input('regtype_id');
       $member->title = $r->input('title');
       $member->firstname = $r->input('firstname');
       $member->lastname = $r->input('lastname');
       $member->email = $r->input('email');
       $member->phone = $r->input('phone');
       $member->dob = $r->input('dob');
       $member->country = $r->input('country');
       $member->state = $r->input('state');
       $member->save();
       return response()->json(['status'=>'success','message'=>'User Edit Successfully']);
    }

    public function deleteMember(Request $r){
        $this->validate($r, [
            'id' => 'required',
            'user_id' => 'required',
        ]);
        $find = Member::where('id',$r->input('id'))->where('action_by',$r->input('user_id'))->first();
        if(is_null($find)){
            return response()->json(['status'=>'error','message'=>'No Member with the provided ID']);
        }
        Member::where('id',$r->input('id'))->where('action_by',$r->input('user_id'))->delete();
        return response()->json(['status'=>'success','message'=>'Member deleted successfully']);
    }

    public function getReports(){
        $members = [];
        $donations = [];
        $conference = [];
      $members['count'] = Member::count();
      $members['sum'] = Member::sum('amount');

      $donations['count'] = Donation::count();
      $donations['sum'] = Donation::sum('amount');

      $conference['count'] = Conference::count();
      $conference['sum'] = Conference::sum('amount');
      return response()->json(['status'=>'success','donations'=>$donations,'members'=>$members,'conference'=>$conference]);
    }

    public function users(){
       $users =  User::all();
       return response()->json(['status'=>'success','users'=>$users]);
    }
     public function conference(){
       $conference =  Conference::paginate(env('PAGINATE',50));
       return response()->json(['status'=>'success','data'=>$conference]);
    }

    public function filterdonation(Request $r){
         $data = Donation::when(!is_null($r->input('name')) && !empty($r->input('name')),function($q) use($r){
            return $q->where('firstname','like','%'.$r->input('name').'%')->orWhere('lastname','like','%'.$r->input('name').'%');
        })->when(!is_null($r->input('phone')) && !empty($r->input('phone')),function($q) use($r){
            return $q->where('phone',$r->input('phone'));
        })->when(!is_null($r->input('date_paid')) && !empty($r->input('date_paid')),function($q) use($r){
            return $q->where('date_paid',date('Y-m-d',strtotime($r->input('date_paid'))));
        })->when(!is_null($r->input('reference')) && !empty($r->input('reference')),function($q) use($r){
            return $q->where('reference',$r->input('reference'));
        })->when(!is_null($r->input('transaction_id')) && !empty($r->input('transaction_id')),function($q) use($r){
            return $q->where('transaction_id',$r->input('transaction_id'));
        })->paginate(env('PAGINATE',50));

        return response()->json(['status'=>'success','data'=>$data]);
    }

    public function filtermembers(Request $r){
        
          $data = Member::with('registrationType')->when(!is_null($r->input('name')) && !empty($r->input('name')),function($q) use($r){
            return $q->where('firstname','like','%'.$r->input('name').'%')->orWhere('lastname','like','%'.$r->input('name').'%');
        })->when(!is_null($r->input('phone')) && !empty($r->input('phone')),function($q) use($r){
            return $q->where('phone',$r->input('phone'));
        })->when(!is_null($r->input('date_paid')) && !empty($r->input('date_paid')),function($q) use($r){
            return $q->where('date_paid',date('Y-m-d',strtotime($r->input('date_paid'))));
        })->when(!is_null($r->input('reference')) && !empty($r->input('reference')),function($q) use($r){
            return $q->where('reference',$r->input('reference'));
        })->when(!is_null($r->input('transaction_id')) && !empty($r->input('transaction_id')),function($q) use($r){
            return $q->where('transaction_id',$r->input('transaction_id'));
        })->when(!is_null($r->input('regtype_id')) && !empty($r->input('regtype_id')),function($q) use($r){
            return $q->where('regtype_id',$r->input('regtype_id'));
        })->paginate(env('PAGINATE',50));

        return response()->json(['status'=>'success','data'=>$data]);

    }


     public function filterconference(Request $r){
         $data = Conference::when(!is_null($r->input('name')) && !empty($r->input('name')),function($q) use($r){
            return $q->where('firstname','like','%'.$r->input('name').'%')->orWhere('surname','like','%'.$r->input('name').'%');
        })->when(!is_null($r->input('phone')) && !empty($r->input('phone')),function($q) use($r){
            return $q->where('phone',$r->input('phone'));
        })->when(!is_null($r->input('date_paid')) && !empty($r->input('date_paid')),function($q) use($r){
            return $q->where('date_paid',date('Y-m-d',strtotime($r->input('date_paid'))));
        })->when(!is_null($r->input('reference')) && !empty($r->input('reference')),function($q) use($r){
            return $q->where('reference',$r->input('reference'));
        })->when(!is_null($r->input('transaction_id')) && !empty($r->input('transaction_id')),function($q) use($r){
            return $q->where('transaction_id',$r->input('transaction_id'));
        })->when(!is_null($r->input('conftype')) && !empty($r->input('conftype')),function($q) use($r){
            return $q->where('conftype',$r->input('conftype'));
        })->paginate(env('PAGINATE',50));

        return response()->json(['status'=>'success','data'=>$data]);
    }
    //
}
