<?php

namespace App\Http\Controllers;
use Illuminate\Support\Facades\Hash;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use App\RegTypes;
use App\Paymentlogs;
use App\Payment;
use App\User;
use Firebase\JWT\JWT;
use Firebase\JWT\ExpiredException;
use App\Member;
use App\Donation;
use App\Conference;
use Barryvdh\DomPDF\Facade as PDF;
//use Maatwebsite\Excel\Facades\Excel;
//use App\Exports\Export;

class MainController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    private $request;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }
    protected function jwt(User $user) {
        $payload = [
            'iss' => "lumen-jwt", // Issuer of the token
            'sub' => $user->id, // Subject of the token
            'iat' => time(), // Time when JWT was issued. 
            'exp' => time() + 60*60*60 // Expiration time
        ];

        return JWT::encode($payload, env('JWT_SECRET'));
    } 
    public function authenticate(User $user) {
        $this->validate($this->request, [
            'email'     => 'required|email',
            'password'  => 'required'
        ]);
        $user = User::where('email', $this->request->input('email'))->first();

        if (!$user) {
            return response()->json([
                'error' => 'Email does not exist.'
            ], 400);
        }
        if (Hash::check($this->request->input('password'), $user->password)) {
            return response()->json([
                'token' => $this->jwt($user),
                'user'=>$user
            ], 200);
        }
        return response()->json([
            'error' => 'Email or password is wrong.'
        ], 400);
    }
    
      public function generatetoken(Request $r){
        $token = $this->generateRandomString(60);
      switch($r->input('type')){
          case 'Membership':
            $this->connectToDB('backendadd')->insert([
                'token'=>$token,
                'form'=>'Membership'
            ]);
            break;
          case 'Donation':
            $this->connectToDB('backendadd')->insert([
                'token'=>$token,
                'form'=>'Donation'
            ]);
            break;
            case 'Conference':
                $this->connectToDB('backendadd')->insert([
                    'token'=>$token,
                    'form'=>'Conference'
                ]);
                break;  
            default:
            break;
      }
      return response()->json(['status'=>'success','token'=>$token]);
    }

    public function getcropinterest(Request $r){
        $data = $this->connectToDB('crop_interest')->get();
        return response()->json(['status' => 'success','data'=>$data]);
    }
    
    public function initpayment(Request $r){
          $find = Paymentlogs::where('reference',$r->input('reference'))->first();
         if(is_null($find)){
             die('Sorry.. payment reference could not be found');
         }else{
             return view('pay',compact('find'));
         }
    }

    public function generateKey(){
        $random = uniqid();
        $paymentlog =  Paymentlogs::where('token',$random)->first();
        if(!is_null($paymentlog)){
            return $this->generateKey();
         } 
         return $random;
    }
    public function generateRandomNumber(){
        $random = mt_rand(1000000000,9999999999);
        $paymentlog =  Paymentlogs::where('reference',$random)->first();
        if(!is_null($paymentlog)){
           return $this->generateRandomNumber();
        } 
   	 return $random;
   }

   public function generateUniqueID(){
    $count = 0;
    $getmax = $this->connectToDB('members')->max('counter');
     if(!is_null($getmax)){
        $count = (int)$getmax;
     }
     $stepup = $count+1;
     $format = 'APBA';
     $extra = '';
     switch (strlen((string)$stepup)) {
       case 1:
         $extra = '000'.(string)$stepup;
         break;
       case 2:
         $extra = '00'.(string)$stepup;
         break;
      case 3:
         $extra = '0'.(string)$stepup;
         break;
      case 4:
         $extra = (string)$stepup;
         break;
       default:
         $extra = (string)$stepup;
         break;
     }
     $finalformat = $format.$extra;
     return ['id'=>$finalformat,'counter'=>$stepup];
   }

    public function generateRandomString($length = 12) {
        return substr(str_shuffle(str_repeat($x='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', ceil($length/strlen($x)) )),1,$length);
   }
    public function connectToDB($table){
        return DB::connection('mysql')->table($table);
    }

    public function RegTypes(){
      $data = RegTypes::get();
      return response()->json(['status' => 'success','data'=>$data]);
    }

    public function getTitles(){
       $data = $this->connectToDB('titles')->get();
       return response()->json(['status' => 'success','data'=>$data]);
    }
    public function getQualification(){
      $data = $this->connectToDB('qualifications')->get();
       return response()->json(['status' => 'success','data'=>$data]);
    }
    public function getSpecialization(){
         $data = $this->connectToDB('specializations')->get();
       return response()->json(['status' => 'success','data'=>$data]);
    }
    public function getCountries(){
        $data = $this->connectToDB('countries')->orderBy('country','ASC')->get();
        return response()->json(['status' => 'success','data'=>$data]);
    }
    public function saveToLogs(Request $r){
        $this->validate($r, [
            'surname' => 'required',
            'firstname' => 'required',
            'email' => 'required',
            'phone'=>'required'
        ]);
       $token = $this->generateKey();
       $reference = $this->generateRandomNumber();
       $log = new Paymentlogs;
       $log->token = $token;
       $log->reference = $reference;
       $log->status = $r->input('statusname');
       $log->amount = $r->input('amount');
       $log->ghcamount = (float)$r->input('amount') * env('RATE');
       $log->member_id = $r->input('member_id');
       $log->currency = $r->input('currency');
       $log->title = $r->input('title');
       $log->surname = $r->input('surname');
       $log->firstname = $r->input('firstname');
       $log->othernames = $r->input('othernames');
       $log->gender = $r->input('gender');
       $log->dob = $r->input('dob');
       $log->country = $r->input('country');
       $log->country_code = $r->input('country_code');
       $log->address = $r->input('address');
       $log->state = $r->input('state');
       $log->email = $r->input('email');
       $log->phone = $r->input('phone');
       $log->paymentfrom = $r->input('paymentfrom');
       $log->allreq = json_encode($r->all());
       if($log->save()){
        //    process payment here and send redirect link
       // return response()->json(['status' => 'success','redirecturl'=>route('pay').'?token='.$token,'token'=>$token,'reference'=>$reference]);
       if($r->input('trojan') != null){
           $gettoken =  $this->connectToDB('backendadd')->where('token',$r->input('trojan'))->first();
           if(is_null($gettoken)){
               return response()->json(['status'=>'success','redirecturl'=>route('initpayment').'?reference='.$log->reference]);
           }else{
               if($log->paymentfrom=='Membership'){
                   $this->saveMembership($log);
                  // $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                   $this->connectToDB('backendadd')->where('token',$r->input('trojan'))->delete();
                    return response()->json(['status' => 'redirect','message'=>'Your Registration went through successfully. Thank you']);
               }
                if($log->paymentfrom=='Donation'){
                   $this->saveDonation($log);
                 //  $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                   $this->connectToDB('backendadd')->where('token',$r->input('trojan'))->delete();
                    return response()->json(['status' => 'redirect','message'=>'Your Donation went through successfully. Thank you']);
               }
               
                 if($log->paymentfrom=='inperson' || $log->paymentfrom=='virtual'){
                   $this->saveConference($log);
                  // $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                   $this->connectToDB('backendadd')->where('token',$r->input('trojan'))->delete();
                    return response()->json(['status' => 'redirect','message'=>'Your Donation went through successfully. Thank you']);
               }
               
               return response()->json(['status'=>'success','redirecturl'=>routexe('initpayment').'?reference='.$log->reference]);
               
           }
       }
       
       elseif( $log->paymentfrom=='virtual'){
            $this->saveConference($log);
            // fire email
            $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference,'virtual');
            return response()->json(['status' => 'success','message'=>'Your Registration went through successfully. Thank you']);
       }
      
        return response()->json(['status'=>'success','redirecturl'=>route('initpayment').'?reference='.$log->reference]);
      // return $this->processPayment($log,$r);
       }
       return response()->json(['status' => 'error']);
    }

    public function processPayment($paymentlog,$r){
            $PROFILE_ID = '73183DA6-1B57-49E8-AF0A-EB1E3F4DC22E';
            $ACCESS_KEY = 'dd6d32d424dd35879e86459c25442126';
            $SECRET_KEY = '7a50cae357fa4e43ba30b47f7f7a64a7cd3d419d39084f13b700ab10d50825ea3ea4fa828d3a44ebbc03f5cd8694fe8cdf739a856468409b9be796f1fd9a051f1adabdcf6d6040d5839b4072b53073e667cb5f6e103c4081a615de1e23672140a2d58521f8c5414c9bf1e905e054b2eb20546cc9b6d14177a06f647229fa53a6';
            $BASE_URL = 'https://secureacceptance.cybersource.com/pay';

            $postData = [
                'profile_id'=>$PROFILE_ID,
                'access_key'=>$ACCESS_KEY,
                'transaction_uuid'=>$paymentlog->token,
                'signed_date_time'=>gmdate("Y-m-d\TH:i:s\Z"),
                'amount'=>$r->input('amount'),
                
               // 'bill_address1'=>$r->input('state','n/a'),
                //'bill_city'=>$r->input('city','n/a'),
                //'bill_country'=>$r->input('country','n/a'),
                //'customer_email'=>$r->input('email','default@mail.com'),
                //'customer_lastname'=>$r->input('firstname','n/a'),
                
                'bill_to_address_line1'=>$r->input('address','n/a'),
                'bill_to_address_city'=>$r->input('state','n/a'),
                'bill_to_address_country'=>$r->input('country_code','GH'),
                'bill_to_email'=>$r->input('email','info_apba@wacci.ug.edu.gh'),
                'bill_to_address_state'=>$r->input('state','n/a'),
                'bill_to_surname'=>$r->input('surname','n/a'),
                'bill_to_forename'=>$r->input('firstname','n/a'),
                
                'locale'=>'en-us',
                'transaction_type'=>'sale',
                'reference_number'=>$paymentlog->reference,
                'currency'=>'USD',
                'override_custom_cancel_page'=> route('response'),
                'override_custom_receipt_page'=> route('response'),
                'signed_field_names'=>'profile_id,access_key,transaction_uuid,signed_date_time,amount,locale,transaction_type,reference_number,currency,override_custom_cancel_page,override_custom_receipt_page,signed_field_names,unsigned_field_names',
                'unsigned_field_names'=>'signature,bill_to_address_city,bill_to_address_country,bill_to_email,bill_to_address_state,bill_to_surname,bill_to_forename,bill_to_address_line1'

            ];
            $signed_data = explode(',',$postData['signed_field_names']);
            $dataToSign = [];
            foreach ($signed_data as $field) {
                $dataToSign[] = $field . "=" . $postData[$field];
            }
            $hidden = '';
            $postData['signature'] = base64_encode(hash_hmac('sha256', implode(',',$dataToSign), $SECRET_KEY, true));
            foreach ($postData as $key => $value) {
                if (strlen($value) > 0){
                $hidden .= '<input type="hidden" name="'.$key.'" value="'.$value.'" />';
             }
            }

            $html =  '
            <html><body onload="document.order.submit()">
                <form name="order" action="'.$BASE_URL.'" method="POST">
                <p>Please wait while your payment is being processed...</p>'.$hidden.'
            </form></body></html>';

              Paymentlogs::where('reference',$paymentlog->reference)->update([
                'signature'=> $postData['signature'],
                'html'=>$html
              ]);
            return response()->json(['status'=>'success','redirecturl'=>route('pay').'?reference='.$paymentlog->reference]);            
               // return;
    }

    public function saveDonation($log){
      $extras = (object)json_decode($log->allreq);
      $donation = new Donation;
      $donation->title = $log->title;
      $donation->firstname = $log->firstname;
      $donation->lastname = $log->surname;
      $donation->email = $log->email;
      $donation->phone = $log->phone;
      $donation->currency = $log->currency;
      $donation->comments = $extras->comments;
      $donation->address = $extras->address;
      $donation->city = isset($extras->city) ? $extras->city : null;
      $donation->state = isset($extras->state) ? $extras->state : null;
      $donation->country = isset($extras->country) ? $extras->country : null;;
      $donation->date_paid = date('Y-m-d');
      $donation->amount = $log->amount;
      $donation->reference = $log->reference;
      $donation->transaction_id = $log->transaction_id;
      $donation->payment_mode = $log->payment_mode;
      if($donation->save()){
        return true;
      }
        return false;
    }

    public function saveConference($log){
        $extras = (object)json_decode($log->allreq);
        $regtype = RegTypes::where('name',$extras->member_type)->first();
        $conf = new Conference;
        $conf->user_id = $extras->member_id;
        $conf->firstname = $log->firstname;
        $conf->surname = $log->surname;
        $conf->othernames = $log->othernames;
        $conf->gender = $log->gender;
        $conf->country = $log->country;
        $conf->country_code = $extras->country_code;
        $conf->state = $log->state;
        $conf->specialization = $extras->specialization;
        $conf->institution = $extras->institution;
        $conf->interest = $extras->interest;
        $conf->qualif = $extras->qualif;
        $conf->email = $log->email;
        $conf->phone = $log->phone;
        $conf->amount = $log->amount;
        $conf->currency = $log->currency;
        $conf->date_paid = date('Y-m-d');
        $conf->reference = $log->reference;
        $conf->transaction_id = $log->transaction_id;
        $conf->payment_mode = $log->payment_mode;
        $conf->conftype = $log->paymentfrom;
        if(!is_null($regtype)){
          $conf->membership_id = $regtype->id;  
        }
        $conf->save();
    }

    public function saveMembership($log){
      $extras = (object)json_decode($log->allreq);
      $regtype = RegTypes::where('name',$extras->statusname)->first();
      $genid = $this->generateUniqueID();
      $member = new Member;
      $member->id = $genid['id'];
      $member->counter = $genid['counter'];
      $member->regtype_id = $regtype->id;
      $member->title = $log->title;
      $member->firstname = $log->firstname;
      $member->lastname = $log->surname;
      $member->gender = $log->gender;
      $member->email = $log->email;
      $member->phone = $log->phone;
      $member->dob = isset($extras->dob) ? $extras->dob : null;
      $member->country = $log->country;
      $member->country_code = isset($extras->country_code) ? $extras->country_code : null;
      $member->state = $log->state;
      $member->specialization = isset($extras->specialization) ? $extras->specialization : null;
      $member->institution = isset($extras->institution) ? $extras->institution :  null;
      $member->interest = isset($extras->interest) ? $extras->interest : null;
      $member->qualif = isset($extras->qualif) ? $extras->qualif : null;
      $member->whatsappno = isset($extras->whatsappno) ? $extras->whatsappno :  null;
      $member->nationality = isset($extras->nationality) ? $extras->nationality :  null;
      $member->address = isset($extras->address) ? $extras->address : null;
      $member->reference = $log->reference;
      $member->transaction_id = $log->transaction_id;
      $member->payment_mode = $log->payment_mode;
      $member->amount = $log->amount;
      $member->currency = $log->currency;
      $member->date_paid = date('Y-m-d');
      $member->save();
      return true;
    }

    public function payResponse(Request $r){
         $redirectto = '';
         $status = 0;
         $log = Paymentlogs::where('reference',$r->input('req_reference_number'))->first();
         if(is_null($log)){
             return redirect(env('CLIENT_BASEURL').'/NotFound');
         }
         Paymentlogs::where('reference',$r->input('req_reference_number'))->update([
            'amount'=>$r->input('req_amount'),
            'currency'=>$r->input('req_currency'),
            'payment_mode'=>$r->input('req_payment_method'),
            'card_number'=>$r->input('req_card_number'),
            'decision'=>$r->input('decision'),
            'message'=>$r->input('message')
       ]);
       $log = Paymentlogs::where('reference',$r->input('req_reference_number'))->first();
         if($r->input('decision')=='ACCEPT' && $r->input('signature')==$log->signature){
           $status = 1;
         }
         if($status==1){
             switch($log->paymentfrom){
                case 'Donation':
                    $this->saveDonation($log);
                   
                    break;
                case 'Membership':
                    $this->saveMembership($log);
                    break;
                case 'inperson':
                case 'virtual':
                    $this->saveConference($log);
                    break;
                default:
                break;
             }
             $redirectto = env('CLIENT_BASEURL').'/thankyou/'.$log->paymentfrom.'?reference='.$log->reference;
         }else{
             switch($log->paymentfrom){
                 case 'Donation':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'Membership':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'inperson':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'virtual':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 default:
                   $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                   break;
             }
         }
         return redirect($redirectto);
    }
    public function getDataWithRef(Request $r){
        $log = Paymentlogs::where('reference',$r->input('reference'))->first([
            'title','firstname','surname','othernames','gender','email','phone','dob','amount','currency','country','state'
        ]);
        return response()->json(['status'=>'success','data'=>$log]);
    }

    public function checkMembership(Request $r){
       $find = Member::with('registrationType')->find(trim($r->input('member_id')));
       if(is_null($find)){
           return response()->json(['status'=>'notfound']);
       }
       $pricing = $this->connectToDB('conference_pricing')->where('name','inperson')->where('regtype',$find->regtype_id)->where('ismember',1)->first();
      
       return response()->json(['status'=>'success','data'=>$find,'pricing'=>$pricing]);
    }

    public function conferencePricing(Request $r){
        if($r->input('type')=='virtual'){
           $fetch = $this->connectToDB('conference_pricing')->where('name',$r->input('type'))->first();  
        }else{
          $fetch = $this->connectToDB('conference_pricing')->where('name',$r->input('type'))->where('regtype',$r->input('regtype'))->where('ismember',0)->first();   
        }
       if(is_null($fetch)){
           return response()->json(['status'=>'error']);
       }
       return response()->json(['status'=>'success','data'=>$fetch]);
    }

    public function pay(Request $r){
         $find = Paymentlogs::where('reference',$r->input('reference'))->first();
         if(is_null($find)){
             die('Sorry.. payment reference could not be found');
         }
         echo $find->html;
         return;
        // return view('pay')->with('data',$find);
    }
 
    public function sendMail($sendto,$subject,$from=null,$ref=null,$template='completed'){
        $data = null;
        switch($from){
            case 'Donation':
                $data = Donation::where('reference',$ref)->first();
                break;
            case 'Membership':
                $data = Member::where('reference',$ref)->first();
                break;
            case 'inperson':
            case 'virtual':
                $data = Conference::where('reference',$ref)->first();
                break;
            default:
            break;
        }
        $add = $this->connectToDB('send_mails')->insert([
            'sendto'=>$sendto,
            'subject'=>$subject,
            'message'=> htmlspecialchars(view('emails.'.$template,compact('data','from'))->render()),
            'type'=>'email'
        ]);
        return true;
    }

    public function exportData($from){
        return 'Downloading...';
        if(is_null($from)){
            return '';
        } 
        $data = null;
        // switch($from){
        //     case 'Donation':
        //         $data = Donation::get();
        //         break;
        //     case 'Membership':
        //         $data = Member::get();
        //         break;
        //     case 'inperson':
        //     case 'virtual':
        //         $data = Conference::get();
        //         break;
        //     default:
        //     break;
        // }
       // return Excel::download(new Export($from), 'dump.xlsx');
    }

    public function downloadInvoice($from=null,$ref=null){
        $invokeprinting = true;
        if(is_null($from)){
            return '';
        } 
        $data = null;
        switch($from){
            case 'Donation':
                $data = Donation::where('reference',$ref)->first();
                break;
            case 'Membership':
                $data = Member::where('reference',$ref)->first();
                break;
            case 'inperson':
            case 'virtual':
                $data = Conference::where('reference',$ref)->first();
                break;
            default:
            break;
        }
        if(is_null($data)){
            return '';
        }
        //dd($data);
        return view('emails.receipt',compact('data','from','invokeprinting'));  
    }
    
     public function quipuCancel(Request $r){
    $resp = $this->parseQuipuResponse($r);
     if(is_null($resp)){
      return redirect(route('initpayment').'?reference=999999999');
     }
    // start
        $redirectto = '';
         $status = 0;
         $log = Paymentlogs::where('quipuOrderID',$resp->OrderID)->where('quipuSessionID',$resp->SessionID)->first();
         if(is_null($log)){
             return redirect(env('CLIENT_BASEURL').'/NotFound');
         }
         //$this->saveDonation($log);
         //$this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
         switch($log->paymentfrom){
            case 'Donation':
               $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
            break;
            case 'Membership':
               $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
            break;
            case 'inperson':
               $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
            break;
            case 'virtual':
               $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
            break;
            default:
              $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
              break;
        }
         return redirect($redirectto);

  }

  public function parseQuipuResponse($r){
    try {
       $xml = base64_decode($r->xmlmsg);
       $xmlRes = new \SimpleXMLElement($xml);
       return (object)['SessionID'=>(string)$xmlRes->SessionID,'OrderID'=>(string)$xmlRes->OrderID,'OrderDescription'=>(string)$xmlRes->OrderDescription];
    } catch (\Exception $e) {
      return null;
    }
  }
  
  public function momoSuccess(Request $r){
       $log = Paymentlogs::where('reference',$r->input('reference'))->first();
         if(is_null($log)){
             return response()->json(['link'=>env('CLIENT_BASEURL').'/NotFound']);
         } 
         
         switch($log->paymentfrom){
            case 'Donation':
                $this->saveDonation($log);
                $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                break;
            case 'Membership':
                $this->saveMembership($log);
                $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                break;
            case 'inperson':
            case 'virtual':
                $this->saveConference($log);
                $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                break;
            default:
            break;
        }
             $redirectto = env('CLIENT_BASEURL').'/thankyou/'.$log->paymentfrom.'?reference='.$log->reference;
             return response()->json(['link'=>$redirectto]);
  }
  
  public function momoCancel(Request $r){
       $log = Paymentlogs::where('reference',$r->input('reference'))->first();
         if(is_null($log)){
             return response()->json(['link'=>env('CLIENT_BASEURL').'/NotFound']);
         } 
       switch($log->paymentfrom){
                 case 'Donation':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'Membership':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'inperson':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'virtual':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 default:
                   $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                   break;
    }
     return response()->json(['link'=>$redirectto]);
  }

  public function quipuSuccess(Request $r){
     $resp = $this->parseQuipuResponse($r);
     if(is_null($resp)){
          die('Sorry.. payment reference could not be found');
      //return redirect(route('initpayment').'?reference=999999999');
     }
    // start  
         $redirectto = '';
         $status = 0;
         if($this->getorderStatus('GetOrderStatus','EN', 'UoG_WACCI',$resp->OrderID,$resp->SessionID)['status']){
             $status = 1;
         }
         $log = Paymentlogs::where('quipuOrderID',$resp->OrderID)->where('quipuSessionID',$resp->SessionID)->first();
         if(is_null($log)){
             return redirect(env('CLIENT_BASEURL').'/NotFound');
         }
    
         if($status==1){
             switch($log->paymentfrom){
                case 'Donation':
                    $this->saveDonation($log);
                    $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                    break;
                case 'Membership':
                    $this->saveMembership($log);
                    $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                    break;
                case 'inperson':
                case 'virtual':
                    $this->saveConference($log);
                    $this->sendMail($log->email,'Application Feedback',$log->paymentfrom,$log->reference);
                    break;
                default:
                break;
             }
             $redirectto = env('CLIENT_BASEURL').'/thankyou/'.$log->paymentfrom.'?reference='.$log->reference;
         }else{
             switch($log->paymentfrom){
                 case 'Donation':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'Membership':
                    $redirectto = env('CLIENT_BASEURL').'/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'inperson':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 case 'virtual':
                    $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                 break;
                 default:
                   $redirectto = env('CLIENT_BASEURL').'/Conference/'.strtolower($log->paymentfrom).'/'.$log->reference;
                   break;
             }
         }
         return redirect($redirectto);
    // end

  }


  public function quipuCheckout(Request $r){
       $orderid = $r->input('reference');
       $log = Paymentlogs::where('reference',$r->input('reference'))->first();
       $merid = 'UoG_WACCI';
       $firstname = $log->firstname;
       $lastname = $log->surname;
       $phone = $log->phone;
       $email = $log->email;
       //$amount=1*100;
       $amount = (float)$log->ghcamount*100;
       //die($amount);
       $link = $this->createOrder('CreateOrder',  //Operation
            'EN',           //Language
            $merid, //Merchant Name
            $amount,         //Amount
            '936',          //Currency
             $log->paymentfrom,  //Order description
             route('quipusuccess'), //ApproveURL - return to this URL in transaction Approved
            route('quipucancel'),  //CancelURL - return to this URL in transaction Canceled
            route('quipucancel'), //DeclineURL - return to this URL in transaction Declined
            'Purchase',
            $email,
            $phone,
            $log->country,
            $log->state,
            $log->address
              ); //Fee 
       if($link != null){
            Paymentlogs::where('reference',$log->reference)->update([
            'quipuOrderID'=>$link['OrderID'],
            'quipuSessionID'=>$link['SessionID'],
          ]);
         $url = $link['url'];
         return redirect($link['url']);
       }else{
       //    var_dump($link);
        return redirect(route('initpayment').'?reference='.$log->reference);
       }

  }

 
    public function sendOverPost($url,$xml){ 
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_SSLKEY, "/home/africanplantbree/quipucert/key.pem");
    curl_setopt($ch, CURLOPT_CAINFO, "/home/africanplantbree/quipucert/ca.pem");
    curl_setopt($ch, CURLOPT_SSLCERT,"/home/africanplantbree/quipucert/cert.pem");
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); //return results
    curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 300);
    $result = curl_exec($ch);
    if(curl_errno($ch))
        print curl_error($ch);
    else
        curl_close($ch);
      return $result;
    }
    
  public function createOrder($operation,$language,$merchant,$amount,$currency,$description,$approveURL,$cancelURL,$declineURL,$orderType,$email,$phone,$country,$state,$address){ 
  $xw = xmlwriter_open_memory();
  xmlwriter_set_indent($xw, 1);
  $res = xmlwriter_set_indent_string($xw, ' ');
  xmlwriter_start_document($xw, '1.0', 'UTF-8');
  xmlwriter_start_element($xw, 'TKKPG');
  xmlwriter_start_element($xw, 'Request');
  xmlwriter_start_element($xw, 'Operation');
  xmlwriter_text($xw, $operation);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'Language');
  xmlwriter_text($xw, $language);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'Order');
  xmlwriter_start_element($xw, 'Merchant');
  xmlwriter_text($xw, $merchant);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'Amount');
  xmlwriter_text($xw, $amount);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'Currency');
  xmlwriter_text($xw, $currency);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'Description');
  xmlwriter_text($xw, $description);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'ApproveURL');
  xmlwriter_text($xw, $approveURL);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'CancelURL');
  xmlwriter_text($xw, $cancelURL);
  xmlwriter_end_element($xw);
  xmlwriter_start_element($xw, 'DeclineURL');
  xmlwriter_text($xw, $declineURL);
  xmlwriter_end_element($xw); 
  
  xmlwriter_start_element($xw, 'OrderType');
  xmlwriter_text($xw, $orderType);
  xmlwriter_end_element($xw);
  
  xmlwriter_start_element($xw, 'AddParams');
	
xmlwriter_start_element($xw, 'FA-DATA');

xmlwriter_text($xw, 'Email='.$email.'; '); 
xmlwriter_text($xw, 'Phone='.$phone.'; '); 
xmlwriter_text($xw, 'ShippingCountry='.$country.'; '); 
xmlwriter_text($xw, 'ShippingCity='.$state.'; '); 
//xmlwriter_text($xw, 'DeliveryPeriod='.$deliveryPeriod.'; '); 
//xmlwriter_text($xw, 'MerchantOrderID='.$merchantExtID.'; '); 
xmlwriter_text($xw, 'ShippingState='.$state.'; '); 
//xmlwriter_text($xw, 'ShippingZipCode='.$shippingZipCode.'; '); 
xmlwriter_text($xw, 'ShippingAddress='.$address.';'); 
xmlwriter_end_element($xw);

xmlwriter_end_element($xw); // AddParams
  
  
  
  xmlwriter_end_element($xw);
  xmlwriter_end_element($xw);
  xmlwriter_end_element($xw);
  xmlwriter_end_document($xw);
  $xmlData = xmlwriter_output_memory($xw);

  // $urlReq = "https://acs2test.quipugmbh.com/Exec";
  $urlReq = "https://mpi.quipugmbh.com:6443/Exec";
  //var_dump($xmlData);
  //die('...');
  $response = $this->sendOverPost($urlReq, $xmlData);
  $xmlRes = new \SimpleXMLElement($response);
  //var_dump($xmlRes);
  //die('..');
  $status = (string)$xmlRes->Response[0]->Status;
  if($status=='00'){
  $orderid = $xmlRes->Response[0]->Order[0]->OrderID;
  $sessionid = $xmlRes->Response[0]->Order[0]->SessionID;
  $url = $xmlRes->Response[0]->Order[0]->URL;
  //generate url to redirect browser
  $redirect = $url . "?ORDERID=" . $orderid . "&SESSIONID=" . $sessionid;
    #dd($redirect);
    return ['url'=>$redirect,'OrderID'=>$orderid,'SessionID'=>$sessionid]; 
  }
  return null;
    
}

public function quipustatus(Request $r){
    return $this->getorderStatus('GetOrderStatus','EN', 'UoG_WACCI',$r->input('orderid'),$r->input('sessionid'));
}

public function getorderStatus($operation,$language,$merchant,$orderid,$sessionid){ 
  $xw1 = xmlwriter_open_memory();
  xmlwriter_set_indent($xw1, 1);
  $res1 = xmlwriter_set_indent_string($xw1, ' ');
  xmlwriter_start_document($xw1, '1.0', 'UTF-8');
  xmlwriter_start_element($xw1, 'TKKPG');
  xmlwriter_start_element($xw1, 'Request');
  xmlwriter_start_element($xw1, 'Operation');
  xmlwriter_text($xw1, $operation);
  xmlwriter_end_element($xw1);
  xmlwriter_start_element($xw1, 'Language');
  xmlwriter_text($xw1, $language); 
  xmlwriter_end_element($xw1);
  xmlwriter_start_element($xw1, 'Order');
  xmlwriter_start_element($xw1, 'Merchant');
  xmlwriter_text($xw1, $merchant);
  xmlwriter_end_element($xw1);
  xmlwriter_start_element($xw1, 'OrderID');
  xmlwriter_text($xw1, $orderid);
  xmlwriter_end_element($xw1);
  xmlwriter_end_element($xw1);
  xmlwriter_start_element($xw1, 'SessionID');
  xmlwriter_text($xw1, $sessionid);
  xmlwriter_end_element($xw1);
  xmlwriter_end_element($xw1);
  xmlwriter_end_element($xw1);
  xmlwriter_end_document($xw1);
  $xmlData1 = xmlwriter_output_memory($xw1);
    $urlReq = "https://mpi.quipugmbh.com:6443/Exec";
    $orderStatus = $this->sendOverPost($urlReq, $xmlData1);
      $xmlRes = new \SimpleXMLElement($orderStatus);
      $status = (string)$xmlRes->Response[0]->Status;
      if($status=='00'){
          return ['status'=>true,'data'=>$orderStatus];
      }
      
    return ['status'=>false,'data'=>$orderStatus]; 
  
}

public function gettransactionstatus(Request $r){
    $url = 'https://digihub.prudentialbank.com.gh/MobileMoneyPayment/api/Transaction/GetTransactionStatus';
    $headers = ['Content-Type: application/json'];
    $details = [
        'clientId'=>env('CLIENT_ID'),
        'walletType'=>$r->input('walletType'),
        'transactionId'=>$r->input('transactionId')
        ];
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($details));
    curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
    curl_setopt($ch, CURLOPT_USERPWD,env('MOMO_USERNAME').':'.env('MOMO_PASSWORD'));
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
    $result = curl_exec($ch);
    $err = curl_error($ch);
    return json_decode(curl_close($ch));
}

public function debitwallet(Request $r){
$url = 'https://digihub.prudentialbank.com.gh/MobileMoneyPayment/api/Transaction/DebitWalletAsync';
$headers = ['Content-Type: application/json'];
$details = [
    'clientId'=>env('CLIENT_ID'),
    'walletType'=>$r->input('walletType'),
    'senderNumber'=>$r->input('senderNumber'),
    'senderName'=>$r->input('senderName'),
    'amount'=>$r->input('amount'),
    'transactionId'=>$r->input('transactionId'),
    'remarks'=>$r->input('remarks')
    ];
$ch = curl_init($url);
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($details));
curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
curl_setopt($ch, CURLOPT_USERPWD,env('MOMO_USERNAME').':'.env('MOMO_PASSWORD'));
curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
curl_setopt($ch, CURLOPT_TIMEOUT, 30);
$result = curl_exec($ch);
$err = curl_error($ch);
curl_close($ch);
//return response()->json(['status'=>'successs','response'=>$result]);
// if (curl_errno($ch)) {
//     return response()->json(['status'=>'error']);
// }else{
//       $info = curl_getinfo($ch);
//       if ($info['http_code'] != 200) {
//         return response()->json(['status'=>'error']);
//       } else {
//          return response()->json(['status'=>'success','response'=>$result]);
//       }
// }
//curl_close($ch);


}


public function walletnameenquiry(Request $r){
$url = 'https://digihub.prudentialbank.com.gh/MobileMoneyPayment/api/Transaction/WalletNameEnquiry';
$headers = ['Content-Type: application/json'];
$details = [
    'clientId'=>env('CLIENT_ID'),
    'walletType'=>$r->input('walletType'),
    'walletNumber'=>$r->input('walletNumber')
    ];
$ch = curl_init($url);
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($details));
curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
curl_setopt($ch, CURLOPT_USERPWD,env('MOMO_USERNAME').':'.env('MOMO_PASSWORD'));
curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
curl_setopt($ch, CURLOPT_TIMEOUT, 30);
$result = curl_exec($ch);
$err = curl_error($ch);
curl_close($ch);
//return response()->json(['status'=>'successs','response'=>$result]);
// if (curl_errno($ch)) {
//     return response()->json(['status'=>'error']);
// }else{
//       $info = curl_getinfo($ch);
//       if ($info['http_code'] != 200) {
//         return response()->json(['status'=>'error']);
//       } else {
//          return response()->json(['status'=>'success','response'=>$result]);
//       }
// }
//curl_close($ch);


}




    //
}
